/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "globals.h"
#include <konkret/konkret.h>
#include "LMI_ElementSettingData.h"
#include <LMI_IPAssignmentSettingData.h>
#include "network.h"
#include "port.h"
#include "connection.h"
#include "endpoint.h"
#include "activeconnection.h"
#include "ref_factory.h"

static const CMPIBroker* _cb;

static void LMI_ElementSettingDataInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_ElementSettingDataCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_ElementSettingDataEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus LMI_ElementSettingDataEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    network_lock(network);
    const Connections *connections = network_get_connections(network);
    if (!connections) {
        KReturn(OK);
    }
    Connection *connection;
    const Endpoints *endpoints;
    Endpoint *endpoint;
    const Ports *allPorts = network_get_ports(network);
    Ports *ports;
    size_t j, k;

    const ActiveConnections *activeConnections = network_get_active_connections(network);
    bool portActive = false;

    for (size_t i = 0; i < connections_length(connections); ++i) {
        connection = connections_index(connections, i);

        CMPIObjectPath *settingDataOP_generic = connectionToCIM_IPAssignmentSettingDataRefOP(connection, _cb, ns);

        // Little hack -- create list of ports for only port we have so it can be processed the same
        if (connection_get_port(connection)) {
            ports = ports_new(1);
            ports_add(ports, (Port *) connection_get_port(connection));
        } else {
            ports = (Ports *) allPorts;
        }

        for (j = 0; j < ports_length(ports); ++j) {
            portActive = active_connections_is_connection_active_on_port(activeConnections, connection, ports_index(ports, j));

            endpoints = network_get_endpoints_by_port(network, ports_index(ports, j));
            for (k = 0; k < endpoints_length(endpoints); ++k) {
                endpoint = endpoints_index(endpoints, k);

                // IP
                CMPIObjectPath *managedElementOP = endpointToLMI_IPProtocolEndpointRefOP(endpoint, _cb, ns);

                LMI_ElementSettingData elementSettingData;
                LMI_ElementSettingData_Init(&elementSettingData, _cb, ns);
                LMI_ElementSettingData_SetObjectPath_ManagedElement(&elementSettingData, managedElementOP);
                LMI_ElementSettingData_SetObjectPath_SettingData(&elementSettingData, settingDataOP_generic);
                LMI_ElementSettingData_Set_IsCurrent(&elementSettingData,
                                                        portActive ?
                                                        LMI_ElementSettingData_IsCurrent_Is_Current :
                                                        LMI_ElementSettingData_IsCurrent_Is_Not_Current);
                LMI_ElementSettingData_Set_IsDefault(&elementSettingData,
                                                        connection_get_autoconnect(connection) ?
                                                        LMI_ElementSettingData_IsDefault_Is_Default :
                                                        LMI_ElementSettingData_IsDefault_Is_Not_Default);
                KReturnInstance(cr, elementSettingData);


                // DNS
                managedElementOP = endpointToLMI_DNSProtocolEndpointRefOP(endpoint, _cb, ns);
                LMI_ElementSettingData_SetObjectPath_ManagedElement(&elementSettingData, managedElementOP);
                KReturnInstance(cr, elementSettingData);
            }
        }
        if (connection_get_port(connection)) {
            // Free only if it has been allocated here
            free(ports);
        }
    }
    network_unlock(network);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_ElementSettingDataGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus LMI_ElementSettingDataCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_ElementSettingDataModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char**properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_ElementSettingDataDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_ElementSettingDataExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static void LMI_ElementSettingDataAssociationInitialize(CMPIAssociationMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_ElementSettingDataAssociationCleanup(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_ElementSettingDataAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_ElementSettingData_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus LMI_ElementSettingDataAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_ElementSettingData_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus LMI_ElementSettingDataReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_ElementSettingData_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus LMI_ElementSettingDataReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_ElementSettingData_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub(
    LMI_ElementSettingData,
    LMI_ElementSettingData,
    _cb,
    LMI_ElementSettingDataInitialize(&mi))

CMAssociationMIStub(
    LMI_ElementSettingData,
    LMI_ElementSettingData,
    _cb,
    LMI_ElementSettingDataAssociationInitialize(&mi))

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_ElementSettingData",
    "LMI_ElementSettingData",
    "instance association")
