/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include <konkret/konkret.h>
#include "LMI_RemoteAccessAvailableToElement.h"
#include <LMI_RemoteServiceAccessPoint.h>
#include "network.h"
#include "activeconnection.h"
#include "connection.h"
#include "setting.h"
#include "endpoint.h"
#include "port.h"
#include "ref_factory.h"

static const CMPIBroker* _cb;

static void LMI_RemoteAccessAvailableToElementInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_RemoteAccessAvailableToElementCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_RemoteAccessAvailableToElementEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus LMI_RemoteAccessAvailableToElementEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    size_t j;
    const char *name;
    char *n;
    Port *port;
    Endpoint *endpoint;

    network_lock(network);
    const Ports *ports = network_get_ports(network);
    for (size_t i = 0; i < ports_length(ports); ++i) {
        port = ports_index(ports, i);
        for (j = 0; j < endpoints_length(port_get_endpoints(port)); ++j) {
            endpoint = endpoints_index(port_get_endpoints(port), j);

            if (endpoint_get_type(endpoint) == Unknown || (endpoint_get_ipv4gateway(endpoint) == NULL && endpoint_get_ipv6gateway(endpoint) == NULL)) {
                continue;
            }


            LMI_RemoteAccessAvailableToElement w;
            LMI_RemoteAccessAvailableToElement_Init(&w, _cb, ns);

            CMPIObjectPath *ipProtocolEndpointOP = endpointToLMI_IPProtocolEndpointRefOP(endpoint, _cb, ns);
            LMI_RemoteAccessAvailableToElement_SetObjectPath_Dependent(&w, ipProtocolEndpointOP);

            LMI_RemoteServiceAccessPointRef rsap;
            LMI_RemoteServiceAccessPointRef_Init(&rsap, _cb, ns);
            LMI_RemoteServiceAccessPointRef_Set_CreationClassName(&rsap, "LMI_RemoteServiceAccessPoint");
            LMI_RemoteServiceAccessPointRef_Set_SystemCreationClassName(&rsap, get_system_creationg_class_name());
            LMI_RemoteServiceAccessPointRef_Set_SystemName(&rsap, get_system_name());

            name = endpoint_get_name(endpoint);
            switch (endpoint_get_type(endpoint)) {
                case IPv4_IPv6:
                    asprintf(&n, "%s IPv4", name);
                    LMI_RemoteServiceAccessPointRef_Set_Name(&rsap, n);
                    LMI_RemoteAccessAvailableToElement_Set_Antecedent(&w, &rsap);
                    KReturnInstance(cr, w);
                    free(n);

                    asprintf(&n, "%s IPv6", name);
                    LMI_RemoteServiceAccessPointRef_Set_Name(&rsap, n);
                    free(n);
                    break;
                default:
                    LMI_RemoteServiceAccessPointRef_Set_Name(&rsap, name);
            }
            LMI_RemoteAccessAvailableToElement_Set_Antecedent(&w, &rsap);
            KReturnInstance(cr, w);
        }
    }
    network_unlock(network);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_RemoteAccessAvailableToElementGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus LMI_RemoteAccessAvailableToElementCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_RemoteAccessAvailableToElementModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char**properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_RemoteAccessAvailableToElementDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_RemoteAccessAvailableToElementExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static void LMI_RemoteAccessAvailableToElementAssociationInitialize(CMPIAssociationMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_RemoteAccessAvailableToElementAssociationCleanup(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_RemoteAccessAvailableToElementAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_RemoteAccessAvailableToElement_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus LMI_RemoteAccessAvailableToElementAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_RemoteAccessAvailableToElement_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus LMI_RemoteAccessAvailableToElementReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_RemoteAccessAvailableToElement_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus LMI_RemoteAccessAvailableToElementReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_RemoteAccessAvailableToElement_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub(
    LMI_RemoteAccessAvailableToElement,
    LMI_RemoteAccessAvailableToElement,
    _cb,
    LMI_RemoteAccessAvailableToElementInitialize(&mi))

CMAssociationMIStub(
    LMI_RemoteAccessAvailableToElement,
    LMI_RemoteAccessAvailableToElement,
    _cb,
    LMI_RemoteAccessAvailableToElementAssociationInitialize(&mi))

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_RemoteAccessAvailableToElement",
    "LMI_RemoteAccessAvailableToElement",
    "instance association")
