/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "globals.h"
#include <konkret/konkret.h>
#include "LMI_StaticIPAssignmentSettingData.h"
#include "network.h"
#include "setting.h"
#include "connection.h"

static const CMPIBroker* _cb = NULL;

static void LMI_StaticIPAssignmentSettingDataInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_StaticIPAssignmentSettingDataCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_StaticIPAssignmentSettingDataEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus LMI_StaticIPAssignmentSettingDataEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    size_t j;
    Connection *connection;
    Setting *setting;
    network_lock(network);
    const Connections *connections = network_get_connections(network);
    for (size_t i = 0; i < connections_length(connections); ++i) {
        connection = connections_index(connections, i);
        for (j = 0; j < settings_length(connection_get_settings(connection)); ++j) {
            setting = settings_index(connection_get_settings(connection), j);
            if (setting_get_type(setting) != SETTING_STATIC && setting_get_type(setting) != SETTING_LINK_LOCAL) {
                continue;
            }

            LMI_StaticIPAssignmentSettingData w;
            LMI_StaticIPAssignmentSettingData_Init(&w, _cb, ns);
            LMI_StaticIPAssignmentSettingData_Set_ElementName(&w, setting_get_id(setting));
            LMI_StaticIPAssignmentSettingData_Set_Caption(&w, setting_get_caption(setting));
            LMI_StaticIPAssignmentSettingData_Set_InstanceID(&w, setting_get_id(setting));
            if (setting_get_protocol_type(setting) == IPv4) {
                LMI_StaticIPAssignmentSettingData_Set_ProtocolIFType(&w, LMI_StaticIPAssignmentSettingData_ProtocolIFType_IPv4);
                LMI_StaticIPAssignmentSettingData_Set_IPv4Address(&w, setting_get_address(setting));
                LMI_StaticIPAssignmentSettingData_Set_SubnetMask(&w, prefixToMask4(setting_get_prefix(setting)));
                LMI_StaticIPAssignmentSettingData_Set_GatewayIPv4Address(&w, setting_get_default_gateway(setting));
            } else {
                LMI_StaticIPAssignmentSettingData_Set_ProtocolIFType(&w, LMI_StaticIPAssignmentSettingData_ProtocolIFType_IPv6);
                LMI_StaticIPAssignmentSettingData_Set_IPv6Address(&w, setting_get_address(setting));
                LMI_StaticIPAssignmentSettingData_Set_IPv6SubnetPrefixLength(&w, setting_get_prefix(setting));
                LMI_StaticIPAssignmentSettingData_Set_GatewayIPv6Address(&w, setting_get_default_gateway(setting));
            }

            KReturnInstance(cr, w);
        }
    }
    network_unlock(network);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_StaticIPAssignmentSettingDataGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus LMI_StaticIPAssignmentSettingDataCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_StaticIPAssignmentSettingDataModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_StaticIPAssignmentSettingDataDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_StaticIPAssignmentSettingDataExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    LMI_StaticIPAssignmentSettingData,
    LMI_StaticIPAssignmentSettingData,
    _cb,
    LMI_StaticIPAssignmentSettingDataInitialize(&mi))

static void LMI_StaticIPAssignmentSettingDataMethodInitialize(CMPIMethodMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_StaticIPAssignmentSettingDataMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_StaticIPAssignmentSettingDataInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return LMI_StaticIPAssignmentSettingData_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    LMI_StaticIPAssignmentSettingData,
    LMI_StaticIPAssignmentSettingData,
    _cb,
    LMI_StaticIPAssignmentSettingDataMethodInitialize(&mi))

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_StaticIPAssignmentSettingData",
    "LMI_StaticIPAssignmentSettingData",
    "instance method")
