/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "network.h"

#include <string.h>
#include <unistd.h>

#include "activeconnection.h"
#include "connection.h"
#include "port.h"

#include "network_private.h"
#include "errors.h"
#include "connection_private.h"

static Network *_network = NULL;
static pthread_once_t network_is_initialized = PTHREAD_ONCE_INIT;

#ifdef DEBUG_ENABLED

#include <signal.h>
#include <sys/wait.h>

void print_backtrace(int signal)
{
    fprintf(stderr, "BackTrace\n");
    int ppid = getpid();
    int pid = fork();
    if (pid == 0) {
        // Child process
        char *strpid;
        asprintf(&strpid, "%d", ppid);
        execl("/usr/bin/gdb", "/usr/bin/gdb", "-p", strpid, NULL);
    } else {
        int status;
        waitpid(pid, &status, 0);
        system("/usr/bin/pkill -9 sfcbd");
    }
}

#endif

Port *network_port_by_mac(Network *network, const char *mac)
{
    Port *port;
    for (size_t i = 0; i < ports_length(network->ports); ++i) {
        port = ports_index(network->ports, i);
        if (strcmp(port_get_mac(port), mac) == 0) {
            return port;
        }
    }
    return NULL;
}

static void *network_thread_start(void *data)
{
    Network *network = data;
    network->priv = network_priv_new(network);
    network->thread_loop = g_main_loop_new(NULL, FALSE);
    // This mutex was locked before calling this function
    MUTEX_UNLOCK(network);

    g_main_loop_run(network->thread_loop);
    g_main_loop_unref(network->thread_loop);
    return NULL;
}

static void network_new()
{
#ifdef DEBUG_ENABLED
    signal(SIGSEGV, print_backtrace);
    signal(SIGABRT, print_backtrace);
    signal(SIGILL, print_backtrace);
    signal(SIGFPE, print_backtrace);
    signal(SIGBUS, print_backtrace);
#endif

    fprintf(stderr, "network_new, pid: %d\n", getpid());
    Network *network;
    network = malloc(sizeof(Network));
    network->loaded = 0;
    network->ref_count = 0;
    network->ports = NULL;
    network->connections = NULL;
    network->activeConnections = NULL;
    pthread_mutex_init(&network->mutex, NULL);
    // Lock the mutex - it will be unlocked after initialization in the background thread
    MUTEX_LOCK(network);
    pthread_create(&network->thread, NULL, network_thread_start, network);
    _network = network;
}

static void network_free(Network *network)
{
    g_main_loop_ref(network->thread_loop);
    g_main_loop_quit(network->thread_loop);
    g_main_loop_unref(network->thread_loop);

    void *res = NULL;
    pthread_join(network->thread, &res);
    pthread_mutex_destroy(&network->mutex);
    free(res);
    if (network->priv) {
        network_priv_free(network->priv);
    }
    if (network->ports) {
        ports_free(network->ports, true);
    }
    if (network->connections) {
        connections_free(network->connections, true);
    }
    if (network->activeConnections) {
        active_connections_free(network->activeConnections, true);
    }
    free(network);
    network = NULL;
}

Network *network_ref(const CMPIBroker *broker)
{
    pthread_once(&network_is_initialized, network_new);

    // Initialization will lock the Network and unlock after it's initialized,
    // so the lock here will wait for initialization in the first run of network_ref
    MUTEX_LOCK(_network);
    _network->ref_count++;
    MUTEX_UNLOCK(_network);
    init_logging(broker);
    return _network;
}

void network_unref(Network *network)
{
    if (network == NULL || _network == NULL) {
        network = _network = NULL;
        return;
    }

    MUTEX_LOCK(network);
    network->ref_count--;

    if (network->ref_count <= 0) {
        network_free(network);
        network_is_initialized = PTHREAD_ONCE_INIT;
        network = _network = NULL;
    } else {
        MUTEX_UNLOCK(network);
    }
}

void network_lock(Network *network)
{
    MUTEX_LOCK(network);
}

void network_unlock(Network *network)
{
    MUTEX_UNLOCK(network);
}

const Ports *network_get_ports(Network *network)
{
    return network->ports;
}

const Endpoints *network_get_endpoints_by_port(Network *network, Port *port)
{
    return port_get_endpoints(port);
}

const Connections *network_get_connections(Network *network)
{
    return network->connections;
}

int network_activate_connection(Network *network, const Port *port, const Connection *connection)
{
    if (connection->port != NULL && !port_compare(port, connection->port)) {
        return LMI_ERROR_CONNECTION_INVALID;
    }
    return network_priv_activate_connection(network, port, connection);
}

const ActiveConnections *network_get_active_connections(Network *network)
{
    return network->activeConnections;
}

int network_create_connection(Network *network, Connection *connection)
{
    return network_priv_create_connection(network, connection);
}

int network_delete_connection(Network *network, Connection *connection)
{
    return network_priv_delete_connection(network, connection);
}

PortStats *network_get_ports_statistics_priv(Network *network, FILE *f)
{
    char *buffer = NULL;
    char *str;
    size_t n = 0;
    ssize_t read;
    Port *port;
    PortStats *stats = port_stats_new(ports_length(network->ports));
    PortStat *stat;

    size_t linenr = 0;
    do {
        // first two lines are headers
        if (linenr < 2) {
            read = getline(&buffer, &n, f);
            if (read == -1) {
                // EOF when reading headers
                if (buffer != NULL) {
                    free(buffer);
                }
                port_stats_free(stats, true);
                fclose(f);
                return NULL;
            }
            linenr++;
            continue;
        }
        // other lines are devices
        read = getdelim(&buffer, &n, ':', f);
        if (read < 1) {
            break;
        }
        // get rid of semicolon at the end
        str = buffer;
        str[read - 1] = '\0';
        // get rid of whilespaces at the beginning
        while (str[0] == ' ') str++;
        port = ports_find_by_id(network->ports, str);
        if (port == NULL) {
            // we don't care about this device, skip the line
            if (getline(&buffer, &n, f) == -1) {
                break;
            }
            continue;
        }

        // read the stats
        if ((read = getline(&buffer, &n, f)) == -1) {
            break;
        }
        stat = port_stat_new();
        if (sscanf(buffer, "%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
               &stat->rx_bytes, &stat->rx_packets, &stat->rx_errs,       &stat->rx_drop,
               &stat->rx_fifo,  &stat->rx_frame,   &stat->rx_compressed, &stat->rx_multicast,
               &stat->tx_bytes, &stat->tx_packets, &stat->tx_errs,       &stat->tx_drop,
               &stat->tx_fifo,  &stat->tx_colls,   &stat->tx_carrier,    &stat->tx_compressed) != 16) {

            break;
        }
        stat->port = port;
        port_stats_add(stats, stat);

        linenr++;
    } while (true);

    if (buffer != NULL) {
        free(buffer);
    }
    fclose(f);
    return stats;
}

PortStats *network_get_ports_statistics(Network *network)
{
    FILE *f = fopen("/proc/net/dev", "r");
    if (f == NULL) {
        error("Unable to open /proc/net/dev for reading");
        return NULL;
    }
    return network_get_ports_statistics_priv(network, f);
}
