/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "port.h"
#include "endpoint.h"
#include "port_private.h"

#include <string.h>
#include <net/if.h>
#include <errno.h>

static int port_read_flags(const char *port_name)
{
    char *flagpath;
    int flags = 0x0;
    asprintf(&flagpath, "/sys/class/net/%s/flags", port_name);
    FILE *f = fopen(flagpath, "r");
    if (f == NULL) {
        error("Unable to open %s: %s", flagpath, strerror(errno));
        return -1;
    }
    if (fscanf(f, "%i", &flags) != 1) {
        fclose(f);
        error("Unable to get the flags for device %s from file %s",
              port_name, flagpath);
        free(flagpath);
        return -2;
    }
    free(flagpath);
    fclose(f);
    return flags;
}

Port *port_new(void)
{
    Port *p = malloc(sizeof(Port));
    p->id = NULL;
    p->uuid = NULL;
    p->mac = NULL;
    p->priv = NULL;
    p->endpoints = NULL;
    return p;
}

void *port_get_priv(const Port *port)
{
    return port->priv;
}

const char *port_get_id(const Port *port)
{
    return port->id;
}

const char *port_get_uuid(const Port *port)
{
    return port->uuid;
}

PortType port_get_type(const Port *port)
{
    return port->type;
}

PortState port_get_state(const Port *port)
{
    int flags = port_read_flags(port_get_id(port));
    if (flags < 0) {
        return STATE_UNKNOWN;
    }
    if (flags & IFF_UP) {
        return STATE_SERVICING;
    } else {
        return STATE_STOPPED;
    }
}

int port_set_state(Port *port, PortState state, GError **err)
{
    int flags = port_read_flags(port_get_id(port));
    if (flags < 0) {
        // Unable to read current flags, assume 0
        flags = 0;
    }
    switch (state) {
        case STATE_STOPPED:
            flags ^= IFF_UP;
            break;
        case STATE_SERVICING:
            flags |= IFF_UP;
            break;
        default:
            return 1;
    }
    char *flagpath;
    asprintf(&flagpath, "/sys/class/net/%s/flags", port_get_id(port));
    FILE *f = fopen(flagpath, "w");
    if (f == NULL) {
        error("Unable to open %s: %s", flagpath, strerror(errno));
        free(flagpath);
        return 2;
    }
    fprintf(f, "0x%x", flags);
    fclose(f);
    return 0;
}

const char *port_get_mac(const Port *port)
{
    return port->mac;
}

const Endpoints *port_get_endpoints(const Port *port)
{
    return port->endpoints;
}

bool port_compare(const Port *p1, const Port *p2)
{
    if (p1 == NULL || p2 == NULL) {
        return false;
    }
    if (p1->id == NULL || p2->id == NULL) {
        return false;
    }
    return strcmp(p1->id, p2->id) == 0;
}

void port_free(Port *p)
{
    if (p->uuid) {
        free(p->uuid);
    }
    if (p->id) {
        free(p->id);
    }
    if (p->mac) {
        free(p->mac);
    }
    if (p->endpoints) {
        endpoints_free(p->endpoints, true);
    }
    port_priv_free(p->priv);
    free(p);
}

LIST_IMPL(Port, port)

Port *ports_find_by_uuid(Ports *ports, const char *uuid)
{
    if (uuid == NULL) {
        return NULL;
    }
    Port *port;
    for (size_t i = 0; i < ports_length(ports); ++i) {
        port = ports_index(ports, i);
        if (port->uuid != NULL && strcmp(port->uuid, uuid) == 0) {
            return port;
        }
    }
    return NULL;
}

Port *ports_find_by_id(Ports *ports, const char *id)
{
    if (id == NULL) {
        return NULL;
    }
    Port *port;
    for (size_t i = 0; i < ports_length(ports); ++i) {
        port = ports_index(ports, i);
        if (port->id != NULL && strcmp(port->id, id) == 0) {
            return port;
        }
    }
    return NULL;
}

LIST_IMPL(PortStat, port_stat)

PortStat *port_stat_new(void)
{
    PortStat *stat = malloc(sizeof(PortStat));
    stat->port = NULL;
    return stat;
}

void port_stat_free(PortStat *stat)
{
    free(stat);
}
