/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "port_nm.h"

#include <string.h>

#include "dbus_wrapper.h"

#include "endpoint_nm.h"
#include "nm_support.h"
#include "network_nm.h"

#include "port_private.h"
#include "endpoint_private.h"
#include "network_private.h"

typedef struct PortPriv {
    DBusGProxy *proxy;
    GHashTable *properties;
    GHashTable *sub_properties;
    Network *network; // Needed for ability to update properties when port state changes
} PortPriv;

void port_state_changed_cb(void *proxy, unsigned int state1, unsigned int state2, unsigned int state3, Port *port);
void port_get_properties(Port *port);
PortState port_state_from_nm_state(uint32_t state);

/* This ugly function is required to handle DBus signal that has 3 uint arguments */
#define g_marshal_value_peek_uint(v)     g_value_get_uint (v)
void
_marshal_VOID__UINT_UINT_UINT (GClosure *closure, GValue *return_value G_GNUC_UNUSED,
        guint n_param_values, const GValue *param_values,
        gpointer invocation_hint G_GNUC_UNUSED, gpointer marshal_data)
{
    typedef void (*GMarshalFunc_VOID__UINT_UINT_UINT) (gpointer data1,
            guint arg_1, guint arg_2, guint arg_3, gpointer data2);
    register GMarshalFunc_VOID__UINT_UINT_UINT callback;
    register GCClosure *cc = (GCClosure *) closure;
    register gpointer data1, data2;

    g_return_if_fail (n_param_values == 4);

    if (G_CCLOSURE_SWAP_DATA(closure))
    {
        data1 = closure->data;
        data2 = g_value_peek_pointer(param_values + 0);
    } else {
        data1 = g_value_peek_pointer(param_values + 0);
        data2 = closure->data;
    }

    callback = (GMarshalFunc_VOID__UINT_UINT_UINT) (size_t) (marshal_data ? marshal_data : cc->callback);
    callback(data1,
             g_marshal_value_peek_uint(param_values + 1),
             g_marshal_value_peek_uint(param_values + 2),
             g_marshal_value_peek_uint(param_values + 3),
             data2);
}

Port *port_new_from_objectpath(Network *network, const char *objectpath)
{
    Port *port = port_new();
    port->uuid = strdup(objectpath);
    PortPriv *priv = malloc(sizeof(PortPriv));
    port->priv = priv;
    priv->network = network;
    priv->proxy = dbus_g_proxy_new_for_name(network_priv_get_dbus_connection(network), NM_SERVICE_DBUS, objectpath, NM_DBUS_INTERFACE_DEVICE);

    port_get_properties(port);

    dbus_g_object_register_marshaller(_marshal_VOID__UINT_UINT_UINT, G_TYPE_NONE, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_INVALID);
    dbus_g_proxy_add_signal(priv->proxy, "StateChanged", G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_INVALID);
    dbus_g_proxy_connect_signal(priv->proxy, "StateChanged", G_CALLBACK(port_state_changed_cb), port, NULL);
    return port;
}

void port_priv_free(void *priv)
{
    PortPriv *p = priv;
    if (p->properties != NULL) {
        g_hash_table_destroy(p->properties);
    }
    if (p->sub_properties != NULL) {
        g_hash_table_destroy(p->sub_properties);
    }
    g_object_unref(p->proxy);
    free(p);
}

void port_read_properties(Port *port);

void port_get_properties(Port *port)
{
    PortPriv *priv = port->priv;
    GHashTable *hash = dbus_get_properties(priv->proxy, NULL, NM_DBUS_INTERFACE_DEVICE);
    if (hash == NULL) {
        error("Unable to get properties for %s", NM_DBUS_INTERFACE_DEVICE);
        return;
    }
    priv->properties = hash;
    port_read_properties(port);
}

void port_create_endpoints(Port *port, const char *ip4config, const char *ip6config)
{
    PortPriv *priv = port->priv;

    Endpoint *endpoint;
    GPtrArray *addresses;
    GHashTable *ipproperties;

    unsigned int id = 0;

    port->endpoints = endpoints_new(0);
    guint32 address, prefix, gateway;

    if (ip4config && strcmp(ip4config, "/") != 0) {
        GArray *array;

        ipproperties = dbus_get_properties(priv->proxy, ip4config, NM_DBUS_INTERFACE_IP4_CONFIG);
        if (ipproperties != NULL) {
            addresses = dbus_property_array(ipproperties, "Addresses");
            if (addresses) {
                for (guint i = 0; i < addresses->len; ++i) {
                    array = (GArray *) g_ptr_array_index(addresses, i);

                    if (!ipv4_address_from_array(array, &address, &prefix, &gateway)) {
                        warn("IPv4 address is invalid");
                        continue;
                    }
                    endpoint = endpoint_new_ipv4(port, address, prefix, gateway);
                    asprintf(&endpoint->name, "%s_%d", port->id, id++);
                    endpoints_add(port->endpoints, endpoint);
                }
            } else {
                warn("No address for Ip4Config on port %s", port->id);
            }
            g_hash_table_destroy(ipproperties);
        } else {
            error("No IPv4 properties on device %s (%s)", port->id, port->uuid);
        }
    }

    // In order to minimize number of endpoint we will try to add both
    // IPv4 and IPv6 address to each endpoint
    size_t index = 0;
    if (ip6config && strcmp(ip6config, "/") != 0) {
        ipproperties = dbus_get_properties(priv->proxy, ip6config, NM_DBUS_INTERFACE_IP6_CONFIG);
        if (ipproperties != NULL) {
            addresses = dbus_property_array(ipproperties, "Addresses");

            if (addresses) {
                GByteArray *address, *gateway;
                guint32 prefix;
                GValueArray *array;

                for (guint i = 0; i < addresses->len; ++i) {
                    array = (GValueArray *) g_ptr_array_index(addresses, i);

                    if (!ipv6_address_from_array(array, &address, &prefix, &gateway)) {
                        warn("IPv6 config (%s) is invalid", ip6config);
                        continue;
                    }

                    if (index < endpoints_length(port->endpoints)) {
                        endpoint = endpoints_index(port->endpoints, index);
                        endpoint_add_ipv6(endpoint, address, prefix, gateway);
                    } else {
                        endpoint = endpoint_new_ipv6(port, address, prefix, gateway);
                        asprintf(&endpoint->name, "%s_%d", port->id, id++);
                        endpoints_add(port->endpoints, endpoint);
                    }
                    index++;
                }
            }
            g_hash_table_destroy(ipproperties);
        } else {
            error("No IPv6 properties on device %s (%s)", port->id, port->uuid);
        }
    }

    if (endpoints_length(port->endpoints) == 0) {
        // Add at least fake endpoint if we don't have any
        endpoint = endpoint_new(port);
        asprintf(&endpoint->name, "%s_%d", port->id, id++);
        endpoint->type = Unknown;
        endpoints_add(port->endpoints, endpoint);
    }
}

void port_read_properties(Port *port)
{
    PortPriv *priv = port->priv;
    const char *iface = dbus_property_string(priv->properties, "Interface");
    if (iface == NULL) {
        error("Device with path %s don't have Interface property", port->uuid);
    } else {
        port->id = strdup(iface);
    }

    port->state = port_state_from_nm_state(dbus_property_uint(priv->properties, "State"));

    const char *ip4config = dbus_property_objectpath(priv->properties, "Ip4Config");
    debug("ip4config: %s", ip4config);
    const char *ip6config = dbus_property_objectpath(priv->properties, "Ip6Config");
    debug("ip6config: %s", ip6config);

    port_create_endpoints(port, ip4config, ip6config);

    // Get device type specific properties
    switch (dbus_property_uint(priv->properties, "DeviceType")) {
        case NM_DEVICE_TYPE_ETHERNET:
            priv->sub_properties = dbus_get_properties(priv->proxy, NULL, NM_DBUS_INTERFACE_DEVICE_WIRED);
            break;
        case NM_DEVICE_TYPE_WIFI:
            priv->sub_properties = dbus_get_properties(priv->proxy, NULL, NM_DBUS_INTERFACE_DEVICE_WIRELESS);
            break;
        case NM_DEVICE_TYPE_BT:
            priv->sub_properties = dbus_get_properties(priv->proxy, NULL, NM_DBUS_INTERFACE_DEVICE_BLUETOOTH);
            break;
        case NM_DEVICE_TYPE_OLPC_MESH:
            priv->sub_properties = dbus_get_properties(priv->proxy, NULL, NM_DBUS_INTERFACE_DEVICE_OLPC_MESH);
            break;
        case NM_DEVICE_TYPE_WIMAX:
            priv->sub_properties = dbus_get_properties(priv->proxy, NULL, NM_DBUS_INTERFACE_DEVICE_WIMAX);
            break;
        case NM_DEVICE_TYPE_MODEM:
            priv->sub_properties = dbus_get_properties(priv->proxy, NULL, NM_DBUS_INTERFACE_DEVICE_MODEM);
            break;
        case NM_DEVICE_TYPE_INFINIBAND:
            priv->sub_properties = dbus_get_properties(priv->proxy, NULL, NM_DBUS_INTERFACE_DEVICE_INFINIBAND);
            break;
        case NM_DEVICE_TYPE_BOND:
            priv->sub_properties = dbus_get_properties(priv->proxy, NULL, NM_DBUS_INTERFACE_DEVICE_BOND);
            break;
        case NM_DEVICE_TYPE_VLAN:
            priv->sub_properties = dbus_get_properties(priv->proxy, NULL, NM_DBUS_INTERFACE_DEVICE_VLAN);
            break;
        default:
            priv->sub_properties = NULL;
            warn("Unknown device type for device %s (%s)", port->id, port->uuid);
    }

    if (priv->sub_properties) {
        // Get MAC address
        const char *mac = dbus_property_string(priv->sub_properties, "HwAddress");
        if (mac) {
            port->mac = strdup(mac);
        } else {
            error("Unknown MAC address for device %s (%s)", port->id, port->uuid);
        }
    }
}

void port_state_changed_cb(void *proxy, unsigned int state1, unsigned int state2, unsigned int state3, Port *port)
{
    debug("Port %s state changed: %d %d %d", port->id, state1, state2, state3);
    PortPriv *priv = port->priv;
    MUTEX_LOCK(priv->network);
    port->state = port_state_from_nm_state(state1);
    port_get_properties(port);
    MUTEX_UNLOCK(priv->network);
}

PortState port_state_from_nm_state(uint32_t state)
{
    debug("Port state: %d", state);
    switch (state) {
        // The device is in an unknown state
        case NM_DEVICE_STATE_UNKNOWN:
            return STATE_UNKNOWN;
            break;
        // The device is recognized but not managed by NetworkManager
        case NM_DEVICE_STATE_UNMANAGED:
            return STATE_NA;
            break;
        // The device cannot be used (carrier off, rfkill, etc)
        case NM_DEVICE_STATE_UNAVAILABLE:
            return STATE_OFFLINE;
            break;
        // The device is not connected
        case NM_DEVICE_STATE_DISCONNECTED:
            return STATE_STOPPED;
            break;
        // The device is preparing to connect
        case NM_DEVICE_STATE_PREPARE:
        // The device is being configured
        case NM_DEVICE_STATE_CONFIG:
        // The device is awaiting secrets necessary to continue connection
        case NM_DEVICE_STATE_NEED_AUTH:
        // The IP settings of the device are being requested and configured
        case NM_DEVICE_STATE_IP_CONFIG:
        // The device's IP connectivity ability is being determined
        case NM_DEVICE_STATE_IP_CHECK:
        // The device is waiting for secondary connections to be activated
        case NM_DEVICE_STATE_SECONDARIES:
            return STATE_STARTING;
            break;
        // The device is active
        case NM_DEVICE_STATE_ACTIVATED:
            return STATE_SERVICING;
            break;
        // The device's network connection is being torn down
        case NM_DEVICE_STATE_DEACTIVATING:
            return STATE_STOPPING;
            break;
        // The device is in a failure state following an attempt to activate it
        case NM_DEVICE_STATE_FAILED:
            return STATE_ABORTED;
            break;
    }
    return STATE_UNKNOWN;
}
