/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "setting.h"

#include <string.h>

#include "port.h"

#include "setting_private.h"

Setting *setting_new(ProtocolType protocol_type, SettingType setting_type)
{
    Setting *setting = malloc(sizeof(Setting));
    setting->id = NULL;
    setting->caption = NULL;
    setting->protocol_type = protocol_type;
    setting->type = setting_type;
    setting->address = NULL;
    setting->prefix = 0;
    setting->default_gateway = NULL;
    setting->clientID = NULL;

    setting->dns_servers = NULL;
    setting->search_domains = NULL;

    return setting;
}

Setting *setting_new_init(ProtocolType protocol_type, SettingType setting_type, const char *address, uint8_t prefix, const char *default_gateway)
{
    Setting *setting = setting_new(protocol_type, setting_type);
    setting->address = strdup(address);
    setting->prefix = prefix;
    if (default_gateway != NULL) {
        setting->default_gateway = strdup(default_gateway);
    } else {
        setting->default_gateway = NULL;
    }
    return setting;
}

SettingType setting_get_type(const Setting *setting)
{
    return setting->type;
}

ProtocolType setting_get_protocol_type(const Setting *setting)
{
    return setting->protocol_type;
}

const char *setting_get_id(const Setting *setting)
{
    return setting->id;
}

const char *setting_get_caption(const Setting *setting)
{
    return setting->caption;
}

const char *setting_get_address(const Setting *setting)
{
    return setting->address;
}

uint8_t setting_get_prefix(const Setting *setting)
{
    return setting->prefix;
}

const char *setting_get_default_gateway(const Setting *setting)
{
    return setting->default_gateway;
}

const char *setting_get_clientID(const Setting *setting)
{
    return setting->clientID;
}

void setting_set_clientID(Setting *setting, const char *clientID)
{
    setting->clientID = strdup(clientID);
}

void setting_add_dns_server(Setting *setting, const char *dns_server)
{
    if (setting->dns_servers == NULL) {
        setting->dns_servers = dns_servers_new(3);
    }
    dns_servers_add(setting->dns_servers, strdup(dns_server));
}

size_t setting_get_dns_servers_length(Setting *setting)
{
    if (setting->dns_servers == NULL) {
        return 0;
    }
    return dns_servers_length(setting->dns_servers);
}

const char *setting_get_dns_server(Setting *setting, size_t index)
{
    if (setting->dns_servers == NULL) {
        return NULL;
    }
    assert(index < dns_servers_length(setting->dns_servers));
    return dns_servers_index(setting->dns_servers, index);
}

void setting_add_search_domain(Setting *setting, const char *search_domain)
{
    if (setting->search_domains == NULL) {
        setting->search_domains = search_domains_new(3);
    }
    search_domains_add(setting->search_domains, strdup(search_domain));
}

size_t setting_get_search_domains_length(Setting *setting)
{
    if (setting->search_domains == NULL) {
        return 0;
    }
    return search_domains_length(setting->search_domains);
}

const char *setting_get_search_domain (Setting *setting, size_t index)
{
    if (setting->search_domains == NULL) {
        return NULL;
    }
    assert(index < search_domains_length(setting->search_domains));
    return search_domains_index(setting->search_domains, index);
}

void setting_free(Setting *setting)
{
    if (setting->id != NULL)
        free(setting->id);
    if (setting->caption != NULL)
        free(setting->caption);
    if (setting->address != NULL)
        free(setting->address);
    if (setting->default_gateway != NULL)
        free(setting->default_gateway);
    if (setting->clientID != NULL)
        free(setting->clientID);
    if (setting->dns_servers != NULL)
        dns_servers_free(setting->dns_servers, true);
    if (setting->search_domains != NULL)
        search_domains_free(setting->search_domains, true);
    free(setting);
}

void dns_server_free(DNSServer *dns)
{
    free(dns);
}

void search_domain_free(SearchDomian *domain)
{
    free(domain);
}


LIST_IMPL(Setting, setting)
LIST_IMPL(DNSServer, dns_server)
LIST_IMPL(SearchDomian, search_domain)
